package de.hft.stuttgart.citydoctor2.healer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.hft.stuttgart.citydoctor2.healing.GeometrySimplifier;
import de.hft.stuttgart.citydoctor2.healing.HealAllPolygonsWrongOrientationError;
import de.hft.stuttgart.citydoctor2.healing.HealConsecutivePointsSameError;
import de.hft.stuttgart.citydoctor2.healing.HealHoleOutsideError;
import de.hft.stuttgart.citydoctor2.healing.HealMainBuilding;
import de.hft.stuttgart.citydoctor2.healing.HealMissingSolid;
import de.hft.stuttgart.citydoctor2.healing.HealNonManifoldEdgeError;
import de.hft.stuttgart.citydoctor2.healing.HealPlanarPolygonError;
import de.hft.stuttgart.citydoctor2.healing.HealPolygonWithoutSurface;
import de.hft.stuttgart.citydoctor2.healing.HealPolygonWrongOrientationError;
import de.hft.stuttgart.citydoctor2.healing.HealRingNotClosedError;
import de.hft.stuttgart.citydoctor2.healing.HealRingSelfIntError;
import de.hft.stuttgart.citydoctor2.healing.HealSameOrientationError;
import de.hft.stuttgart.citydoctor2.healing.HealSolidNotClosedError;
import de.hft.stuttgart.citydoctor2.healing.HealTooFewPoints;
import de.hft.stuttgart.citydoctor2.healing.HealWrongDormers;

public class HealingMethods {
	
	private static List<HealingMethodPrototype> healingMethods;
	
	private HealingMethods() {
		// only static access
	}
	
	static {
		healingMethods = new ArrayList<>();
		healingMethods.add(HealingMethodPrototype.of(new HealMissingSolid()));
		healingMethods.add(HealingMethodPrototype.of(new HealMainBuilding()));
		healingMethods.add(HealingMethodPrototype.of(new HealConsecutivePointsSameError()));
		healingMethods.add(HealingMethodPrototype.of(new HealTooFewPoints()));
		healingMethods.add(HealingMethodPrototype.of(new HealRingNotClosedError()));
		healingMethods.add(HealingMethodPrototype.of(new HealRingSelfIntError()));
		healingMethods.add(HealingMethodPrototype.of(new HealPlanarPolygonError()));
//		healingMethods.add(HealingMethodPrototype.of(new HealPlanarPolygonCpp()));
		healingMethods.add(HealingMethodPrototype.of(new HealSameOrientationError()));
		healingMethods.add(HealingMethodPrototype.of(new HealHoleOutsideError()));
		healingMethods.add(HealingMethodPrototype.of(new HealNonManifoldEdgeError()));
		healingMethods.add(HealingMethodPrototype.of(new HealSolidNotClosedError()));
//		healingMethods.add(HealingMethodPrototype.of(new HealSolidNotClosedCpp()));
		healingMethods.add(HealingMethodPrototype.of(new HealPolygonWrongOrientationError()));
		healingMethods.add(HealingMethodPrototype.of(new GeometrySimplifier()));
		healingMethods.add(HealingMethodPrototype.of(new HealWrongDormers()));
		healingMethods.add(HealingMethodPrototype.of(new HealAllPolygonsWrongOrientationError()));
		healingMethods.add(HealingMethodPrototype.of(new HealPolygonWithoutSurface()));
		healingMethods = Collections.unmodifiableList(healingMethods);
	}
	
	public static HealingPlan createHealingPlanFromAllMethods() {
		HealingPlan plan = new HealingPlan();
		for (HealingMethodPrototype proto : healingMethods) {
			plan.addHealingMethod(proto.createMethod());
		}
		return plan;
	}
	
	public static List<HealingMethodPrototype> getAvailableHealingMethods() {
		return healingMethods;
	}

}
